; LCARS/theme.jl

;; Copyright (C) 1999 Ryan Lovett <ryan@ocf.berkeley.edu>

;; This theme is based on the computer interfaces from Star Trek
;; By Ryan Lovett (ryan@ocf.berkeley.edu).
(require 'gradient)

(defgroup LCARS-frame "LCARS frame")

(defcustom LCARS:window-ops-side 'left
  "The side of the window the window operation buttons are attached to."
  :type symbol
  :options (left right)
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:justify-title 'right
  "Title justification."
  :type symbol
  :options (left center right)
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:color-bg-title (get-color "#9999ff")
  "Title bar background color."
  :type color
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:color-fg-title (get-color "#000000")
  "Title bar foreground color."
  :type color
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:color-bg-menu (get-color "#cc99ff")
  "Menu button background color."
  :type color
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:color-fg-menu (get-color "#000000")
  "Menu button foreground color."
  :type color
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:color-bg-ops (get-color "#ff9900")
  "Operations buttons background color."
  :type color
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:color-fg-ops (get-color "#000000")
  "Operations buttons foreground color."
  :type color
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:element-size 18
  "Uniform element size."
  :type number
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:element-sep 2
  "Separation between elements."
  :type number
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:element-op-width 60
  "Operations buttons width."
  :type number
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:order-menu 0
  "Menu button slot."
  :type number
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:order-close 1
  "Close button slot."
  :type number
  :group LCARS-frame
  :after-set after-setting-frame-option)

(defcustom LCARS:order-iconify 2
  "Iconify button slot."
  :type number
  :group LCARS-frame
  :after-set after-setting-frame-option)



(let*
    
    (
     ;; Font
     (font
      (condition-case nil
	  (get-font "-eraman-haettenschweiler-medium-r-normal-*-16-*-*-*-p-*-iso8859-15")
	(error
	 (condition-case nil
	     (get-font "-b&h-lucida-bold-r-normal-*-*-120-*-*-p-*-iso8859-1")
	   (error default-font)))))


     ;; Update window title pixel length
     (title-width
      (lambda (w)
	(let
	    ((w-width (car (window-dimensions w))))
	  (max 0 (min (- w-width 100) (text-width (window-name w) font))))))
     
     


     ;; 20x20
     (image-left-oval (make-image "oval-left.png"))
     
     ;; 20x20
     (image-right-oval (make-image "oval-right.png"))
     
     ;; black
     (color-black (get-color "#000000"))

     ;; orange
     (color-orange (get-color "#ff9900"))

     ;; yellow
     (color-yellow (get-color "#ffcc00"))

     ;; light blue
     (color-light-blue (get-color "#9999ff"))

     ;; light purple
     (color-light-purple (get-color "#cc99ff"))

     ;; dark purple
     (color-dark-purple (get-color "#6600cc"))
      
     ;; medium purple
     (color-medium-purple (get-color "#996699"))
      
     ;; frame layout
     (frame-left 
      `(
	(;; menu
	 (background . ,LCARS:color-bg-menu)
	 (foreground . ,LCARS:color-fg-menu)
	 (text . "Menu")
	 (x-justify . -4)
	 (y-justify . center)
	 (font . ,font)
	 (left-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-menu 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . menu-button))

	(;; close
	 (background . ,LCARS:color-bg-ops)
	 (foreground . ,LCARS:color-fg-ops)
	 (text . "Close")
	 (x-justify . -4)
	 (y-justify . center)
	 (font . ,font)
	 (left-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-close 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . close-button))

	(;; iconify
	 (background . ,LCARS:color-bg-ops)
	 (foreground . ,LCARS:color-fg-ops)
	 (text . "Iconify")
	 (x-justify . -4)
	 (y-justify . center)
	 (font . ,font)
	 (left-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-iconify 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . iconify-button))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (left-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
	                (- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; title
	 (background . ,LCARS:color-bg-title)
	 (foreground . ,LCARS:color-fg-title)
	 (font . ,font)
	 (text . ,window-name)
	 (x-justify . ,LCARS:justify-title)
	 (y-justify . center)
	 (left-edge . 4)
	 (right-edge . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (right-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; thumb
	 (background . ,image-right-oval)
	 (renderer . ,(lambda (img) (apply draw-horizontal-gradient
					   img (list LCARS:color-bg-title
						     LCARS:color-bg-title))))
	 (right-edge . ,(lambda (w) (- 0 (+ LCARS:element-sep
					   LCARS:element-size))))
	 (width . ,LCARS:element-size)
	 (height . ,LCARS:element-size)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (class . top-right-corner))))


     (frame-right 
      `(
	(;; menu
	 (background . ,LCARS:color-bg-menu)
	 (foreground . ,LCARS:color-fg-menu)
	 (text . "Menu")
	 (x-justify . +4)
	 (y-justify . center)
	 (font . ,font)
	 (right-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-menu 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . menu-button))

	(;; close
	 (background . ,LCARS:color-bg-ops)
	 (foreground . ,LCARS:color-fg-ops)
	 (text . "Close")
	 (x-justify . +4)
	 (y-justify . center)
	 (font . ,font)
	 (right-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-close 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . close-button))

	(;; iconify
	 (background . ,LCARS:color-bg-ops)
	 (foreground . ,LCARS:color-fg-ops)
	 (text . "Iconify")
	 (x-justify . +4)
	 (y-justify . center)
	 (font . ,font)
	 (right-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-iconify 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . iconify-button))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (left-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; title
	 (background . ,LCARS:color-bg-title)
	 (foreground . ,LCARS:color-fg-title)
	 (font . ,font)
	 (text . ,window-name)
	 (x-justify . ,LCARS:justify-title)
	 (y-justify . center)
	 (left-edge . 4)
	 (right-edge . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (right-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))
	      
	(;; thumb
	 (background . ,image-left-oval)
	 (renderer . ,(lambda (img) (apply draw-horizontal-gradient
					   img (list LCARS:color-bg-title
						     LCARS:color-bg-title))))
	 (left-edge . ,(lambda (w) (- 0 (+ LCARS:element-sep
					   LCARS:element-size))))
	 (width . ,LCARS:element-size)
	 (height . ,LCARS:element-size)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (class . top-left-corner))))

     ;; frame layout
     (frame-s-left 
      `(
	(;; menu
	 (background . ,LCARS:color-bg-menu)
	 (foreground . ,LCARS:color-fg-menu)
	 (text . "Menu")
	 (x-justify . -4)
	 (y-justify . center)
	 (font . ,font)
	 (left-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-menu 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . menu-button))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (left-edge . 0)
	 (width . 4)
	 (top-edge . -22)
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; title
	 (background . ,LCARS:color-bg-title)
	 (foreground . ,LCARS:color-fg-title)
	 (font . ,font)
	 (text . ,window-name)
	 (x-justify . ,LCARS:justify-title)
	 (y-justify . center)
	 (left-edge . 4)
	 (right-edge . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (right-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; thumb
	 (background . ,image-right-oval)
	 (renderer . ,(lambda (img) (apply draw-horizontal-gradient
					   img (list LCARS:color-bg-title
						     LCARS:color-bg-title))))
	 (right-edge . ,(lambda (w) (- 0 (+ LCARS:element-sep
					   LCARS:element-size))))
	 (width . ,LCARS:element-size)
	 (height . ,LCARS:element-size)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (class . top-right-corner))))


     (frame-s-right 
      `(
	(;; menu
	 (background . ,LCARS:color-bg-menu)
	 (foreground . ,LCARS:color-fg-menu)
	 (text . "Menu")
	 (x-justify . +4)
	 (y-justify . center)
	 (font . ,font)
	 (right-edge . ,(lambda (w) (- 0 LCARS:element-op-width)))
	 (width . ,(lambda (w) (- LCARS:element-op-width LCARS:element-sep)))
	 (top-edge . ,(lambda (w) (* (- LCARS:order-menu 1)
				     (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . menu-button))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (left-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; title
	 (background . ,LCARS:color-bg-title)
	 (foreground . ,LCARS:color-fg-title)
	 (font . ,font)
	 (text . ,window-name)
	 (x-justify . ,LCARS:justify-title)
	 (y-justify . center)
	 (left-edge . 4)
	 (right-edge . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))

	(;; margin
	 (background . ,LCARS:color-bg-title)
	 (right-edge . 0)
	 (width . 4)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (height . ,LCARS:element-size)
	 (class . title))
	      
	(;; thumb
	 (background . ,image-left-oval)
	 (renderer . ,(lambda (img) (apply draw-horizontal-gradient
					   img (list LCARS:color-bg-title
						     LCARS:color-bg-title))))
	 (left-edge . ,(lambda (w) (- 0 (+ LCARS:element-sep
					   LCARS:element-size))))
	 (width . ,LCARS:element-size)
	 (height . ,LCARS:element-size)
	 (top-edge . ,(lambda (w)
			(- 0 (+ LCARS:element-sep LCARS:element-size))))
	 (class . top-left-corner))))
     )


  
  (add-frame-style 'LCARS
		   (lambda (w type)
		     (cond
		      ((eq type 'shaped)
		       (cond ((eq LCARS:window-ops-side 'left)
			      frame-s-left)
			     ((eq LCARS:window-ops-side 'right)
			      frame-s-right)))
		      ((eq type 'unframed)
		       nil-frame)	   
		      (t
		       (cond ((eq LCARS:window-ops-side 'left)
			      frame-left)
			     ((eq LCARS:window-ops-side 'right)
			      frame-right))))))
     
  (call-after-property-changed
   'WM_NAME (lambda ()
	      (rebuild-frames-with-style 'LCARS))))



